@@ -481,32 +481,33 @@
 	 (do ((l (cdr e) (cdr l)) (c 0 (+ c ($nterms (car l)))))
 	     ((null l) c)))
 	((and (eq (caar e) 'mexpt) (integerp (caddr e)) (plusp (caddr e)))
-	 ($binomial (+ (caddr e) ($nterms (cadr e)) -1) (caddr e)))
+	 (ftake '%binomial (+ (caddr e) ($nterms (cadr e)) -1) (caddr e)))
 	((specrepp e) ($nterms (specdisrep e)))
 	(t 1)))
 
 ;;;; ATAN2
 
 ;; atan2 distributes over lists, matrices, and equations
-(defprop $atan2 (mlist $matrix mequal) distribute_over)
+(defprop %atan2 (mlist $matrix mequal) distribute_over)
 
-(defun simpatan2 (expr vestigial z)     ; atan2(y,x) ~ atan(y/x)
-  (declare (ignore vestigial))
-  (twoargcheck expr)
-  (let (y x signy signx)
-    (setq y (simpcheck (cadr expr) z)
-          x (simpcheck (caddr expr) z))
+;; I have locally set `limitp` to `nil` in two calls to `$sign`. This change
+;; resolves bug #4529: `atan2` limit issue with `radexpand: false`.
+;; Specifically, it resolves the bug when computing the limit:
+;; `block([radexpand: false], limit(atan2(x^2 - 2, x^3 - 3*x), x, sqrt(2), minus))`.
+;; Arguably, this fix is inelegant and should be revisited. (Barton Willis, April 2025)
+(def-simplifier atan2 (y x)
+  (let (signy signx)
     (cond ((and (zerop1 y) (zerop1 x))
            (merror (intl:gettext "atan2: atan2(0,0) is undefined.")))
-          ( ;; float contagion
-           (and (or (numberp x) (ratnump x))       ; both numbers
-                (or (numberp y) (ratnump y))       ; ... but not bigfloats
+          (;; float contagion
+           (and (or (numberp x) (ratnump x)) ; both numbers
+                (or (numberp y) (ratnump y)) ; ... but not bigfloats
                 (or $numer (floatp x) (floatp y))) ; at least one float
            (atan ($float y) ($float x)))
-          ( ;; bfloat contagion
+          (;; bfloat contagion
            (and (mnump x)
                 (mnump y)
-                (or ($bfloatp x) ($bfloatp y)))    ; at least one bfloat
+                (or ($bfloatp x) ($bfloatp y)))	; at least one bfloat
            (setq x ($bfloat x)
                  y ($bfloat y))
            (*fpatan y (list x)))
@@ -517,12 +518,14 @@
            0)
           ((or (eq x '$minf)
                (alike1 x '((mtimes) -1 $inf)))
-           ;; Simplify atan2(y,minf) -> %pi for realpart(y)>=0 or 
-           ;; -%pi for realpart(y)<0. When sign of y unknwon, return noun form.
-           (cond ((member (setq signy ($sign ($realpart x))) '($pos $pz $zero)) 
+           ;; Simplify atan2(y,minf) -> %pi for realpart(y)>=0 or -%pi
+           ;; for realpart(y)<0. When sign of y unknwon, return noun
+           ;; form.  We are basically making atan2 on the branch cut
+           ;; be continuous with quadrant II.
+           (cond ((member (setq signy ($sign ($realpart y))) '($pos $pz $zero)) 
                   '$%pi)
                  ((eq signy '$neg) (mul -1 '$%pi))
-                 (t (eqtest (list '($atan2) y x) expr))))
+                 (t (give-up))))
           ((or (eq y '$inf)
                (alike1 y '((mtimes) -1 $minf)))
            ;; Simplify atan2(inf,x) -> %pi/2
@@ -531,26 +534,38 @@
                (alike1 y '((mtimes -1 $inf))))
            ;; Simplify atan2(minf,x) -> -%pi/2
            (div '$%pi -2))
-          ((and (free x '$%i) (setq signx ($sign x))
-                (free y '$%i) (setq signy ($sign y))
+          ((and (free x '$%i) (setq signx (let ((limitp nil)) ($sign x)))
+                (free y '$%i) (setq signy (let ((limitp nil)) ($sign y)))
                 (cond ((zerop1 y)
-                       (cond ((eq signx '$neg) '$%pi)
+                       ;; Handle atan2(0, x) which is %pi or -%pi
+                       ;; depending on the sign of x.  We assume that
+                       ;; x is never actually zero since atan2(0,0) is
+                       ;; undefined.
+                       (cond ((member signx '($neg $nz)) '$%pi)
                              ((member signx '($pos $pz)) 0)))
                       ((zerop1 x)
+                       ;; Handle atan2(y, 0) which is %pi/2 or -%pi/2,
+                       ;; depending on the sign of y.
                        (cond ((eq signy '$neg) (div '$%pi -2))
                              ((member signy '($pos $pz)) (div '$%pi 2))))
                       ((alike1 y x)
+                       ;; Handle atan2(x,x) which is %pi/4 or -3*%pi/4
+                       ;; depending on the sign of x.
                        (cond ((eq signx '$neg) (mul -3 (div '$%pi 4)))
                              ((member signx '($pos $pz)) (div '$%pi 4))))
                       ((alike1 y (mul -1 x))
+                       ;; Handle atan2(-x,x) which is 3*%pi/4 or
+                       ;; -%pi/4 depending on the sign of x.
                        (cond ((eq signx '$neg) (mul 3 (div '$%pi 4)))
                              ((member signx '($pos $pz)) (div '$%pi -4)))))))
           ($logarc
            (logarc '%atan2 (list ($logarc y) ($logarc x))))
           ((and $trigsign (eq t (mminusp y)))
-           (neg (take '($atan2) (neg y) x)))
+           ;; atan2(-y,x) = -atan2(y,x) if trigsign is true.
+           (neg (take '(%atan2) (neg y) x)))
           ;; atan2(y,x) = atan(y/x) + pi sign(y) (1-sign(x))/2
           ((eq signx '$pos)
+           ;; atan2(y,x) = atan(y/x) when x is positive.
            (take '(%atan) (div y x)))
           ((and (eq signx '$neg)
                 (member (setq signy ($csign y)) '($pos $neg) :test #'eq))
@@ -560,7 +575,7 @@
            ;; Unfortunately, we'll rarely get here.  For example,
            ;; assume(equal(x,0)) atan2(x,x) simplifies via the alike1 case above
            (merror (intl:gettext "atan2: atan2(0,0) is undefined.")))
-          (t (eqtest (list '($atan2) y x) expr)))))
+          (t (give-up)))))
 
 ;;;; ARITHF
 
@@ -646,12 +661,12 @@
     (resimplify (apply #'scanmap1 (mmapev l)))))
 
 (defun scanmap1 (func e &optional (flag nil flag?))
+  (when (and flag? (not (member flag '($bottomup $topdown))))
+    (merror (intl:gettext "scanmap: third argument must be 'bottomup' or 'topdown', if present; found ~M") flag))
   (let ((arg2 (specrepcheck e)) newarg2)
     (cond ((eq func '$rat)
 	   (merror (intl:gettext "scanmap: cannot apply 'rat'.")))
-	  (flag?
-	   (unless (eq flag '$bottomup)
-	     (merror (intl:gettext "scanmap: third argument must be 'bottomup', if present; found ~M") flag))
+	  ((eq flag '$bottomup)
 	   (if (mapatom arg2)
 	       (funcer func (ncons arg2))
 	       (subst0 (funcer func
@@ -798,9 +813,11 @@
 	 (cons (cons (getopr ary) '(array)) (cdr subs)))
 	(t (cons '(mqapply array) (cons ary (cdr subs))))))
 
-(defmspec $arrayinfo (ary)
-  (setq ary (cdr ary))
-  (arrayinfo-aux (car ary) (getvalue (car ary))))
+(defmspec $arrayinfo (e)
+  (let*
+    ((ary (second e))
+     (ary-value (if (or (arrayp ary) (hash-table-p ary)) ary (getvalue ary))))
+    (arrayinfo-aux ary ary-value)))
 
 (defun arrayinfo-aux (sym val)
   (prog (arra ary)
